diff --git a/core/lib/Drupal/Core/Entity/DatabaseStorageControllerNG.php b/core/lib/Drupal/Core/Entity/DatabaseStorageControllerNG.php
index 02b9978..7c5c7ae 100644
--- a/core/lib/Drupal/Core/Entity/DatabaseStorageControllerNG.php
+++ b/core/lib/Drupal/Core/Entity/DatabaseStorageControllerNG.php
@@ -214,10 +214,10 @@ protected function buildQuery($ids, $revision_id = FALSE) {
    *
    * Added mapping from storage records to entities.
    */
-  protected function attachLoad(&$queried_entities, $load_revision = FALSE) {
+  protected function postLoad(&$queried_entities, $revision_id = FALSE) {
     // Map the loaded stdclass records into entity objects and according fields.
-    $queried_entities = $this->mapFromStorageRecords($queried_entities, $load_revision);
-    parent::attachLoad($queried_entities, $load_revision);
+    $queried_entities = $this->mapFromStorageRecords($queried_entities, $revision_id);
+    parent::postLoad($queried_entities, $revision_id);
   }
 
   /**
@@ -225,13 +225,14 @@ protected function attachLoad(&$queried_entities, $load_revision = FALSE) {
    *
    * @param array $records
    *   Associative array of query results, keyed on the entity ID.
-   * @param boolean $load_revision
-   *   (optional) TRUE if the revision should be loaded, defaults to FALSE.
+   * @param int $revision_id
+   *   ID of the revision that was loaded, or FALSE if the most current revision
+   *   was loaded.
    *
    * @return array
    *   An array of entity objects implementing the EntityInterface.
    */
-  protected function mapFromStorageRecords(array $records, $load_revision = FALSE) {
+  protected function mapFromStorageRecords(array $records, $revision_id = FALSE) {
     $entities = array();
     foreach ($records as $id => $record) {
       $entities[$id] = array();
@@ -249,7 +250,7 @@ protected function mapFromStorageRecords(array $records, $load_revision = FALSE)
         $entities[$id] = new $this->entityClass($entities[$id], $this->entityType, $bundle);
       }
     }
-    $this->attachPropertyData($entities, $load_revision);
+    $this->attachPropertyData($entities, $revision_id);
     return $entities;
   }
 
diff --git a/core/lib/Drupal/Core/Entity/Entity.php b/core/lib/Drupal/Core/Entity/Entity.php
index 6d41d13..3a24fbe 100644
--- a/core/lib/Drupal/Core/Entity/Entity.php
+++ b/core/lib/Drupal/Core/Entity/Entity.php
@@ -642,9 +642,36 @@ public static function postDelete(EntityStorageControllerInterface $storage_cont
   }
 
   /**
+   * Collects information for hook load.
+   *
+   * @param array $entities
+   *   An array of entities.
+   *
+   * @return array
+   *   A list of arguments to be passed by postLoad() to the entity load
+   *   hooks.
+   */
+  protected static function hookLoadArguments(array $entities) {
+    return array();
+  }
+
+  /**
    * {@inheritdoc}
    */
-  public static function postLoad(EntityStorageControllerInterface $storage_controller, array $entities) {
+  public static function postLoad(EntityStorageControllerInterface $storage_controller, array $entities, $revision_id = FALSE) {
+    $entity_type = $storage_controller->entityType();
+    // Call hook_entity_load().
+    foreach (\Drupal::moduleHandler()->getImplementations('entity_load') as $module) {
+      $function = $module . '_entity_load';
+      $function($entities, $entity_type);
+    }
+    // Call hook_TYPE_load(). The first argument for hook_TYPE_load() are
+    // always the queried entities, followed by additional arguments set in
+    // $this->hookLoadArguments
+    $args = array_merge(array($entities), static::hookLoadArguments($entities));
+    foreach (\Drupal::moduleHandler()->getImplementations($entity_type . '_load') as $module) {
+      call_user_func_array($module . '_' . $entity_type . '_load', $args);
+    }
   }
 
   /**
diff --git a/core/lib/Drupal/Core/Entity/EntityInterface.php b/core/lib/Drupal/Core/Entity/EntityInterface.php
index c0949f7..143b822 100644
--- a/core/lib/Drupal/Core/Entity/EntityInterface.php
+++ b/core/lib/Drupal/Core/Entity/EntityInterface.php
@@ -236,14 +236,17 @@ public static function preDelete(EntityStorageControllerInterface $storage_contr
   public static function postDelete(EntityStorageControllerInterface $storage_controller, array $entities);
 
   /**
-   * Acts on loaded entities before the load hook is invoked.
+   * Calls the entity load hooks.
    *
    * @param EntityStorageControllerInterface $storage_controller
    *   The entity storage controller object.
    * @param array $entities
    *   An array of entities.
+   * @param int|bool $revision_id
+   *   ID of the revision that was loaded, or FALSE if the most current revision
+   *   was loaded.
    */
-  public static function postLoad(EntityStorageControllerInterface $storage_controller, array $entities);
+  public static function postLoad(EntityStorageControllerInterface $storage_controller, array $entities, $revision_id = FALSE);
 
   /**
    * Creates a duplicate of the entity.
diff --git a/core/lib/Drupal/Core/Entity/EntityStorageControllerBase.php b/core/lib/Drupal/Core/Entity/EntityStorageControllerBase.php
index 43ff5f1..f7f5938 100644
--- a/core/lib/Drupal/Core/Entity/EntityStorageControllerBase.php
+++ b/core/lib/Drupal/Core/Entity/EntityStorageControllerBase.php
@@ -45,15 +45,6 @@
   protected $entityInfo;
 
   /**
-   * Additional arguments to pass to hook_TYPE_load().
-   *
-   * Set before calling Drupal\Core\Entity\DatabaseStorageController::attachLoad().
-   *
-   * @var array
-   */
-  protected $hookLoadArguments = array();
-
-  /**
    * Name of the entity's ID field in the entity database table.
    *
    * @var string
@@ -87,6 +78,20 @@ public function __construct($entity_type, $entity_info) {
   /**
    * {@inheritdoc}
    */
+  public function entityType() {
+    return $this->entityType;
+  }
+
+  /**
+   * {@inheritdoc}
+   */
+  public function entityInfo() {
+    return $this->entityInfo;
+  }
+
+  /**
+   * {@inheritdoc}
+   */
   public function loadUnchanged($id) {
     $this->resetCache(array($id));
     return $this->load($id);
@@ -272,4 +277,18 @@ protected function invokeTranslationHooks(EntityInterface $entity) {
     }
   }
 
+  /**
+   * Attaches data to entities upon loading.
+   *
+   * @param $queried_entities
+   *   Associative array of query results, keyed on the entity ID.
+   * @param $revision_id
+   *   ID of the revision that was loaded, or FALSE if the most current revision
+   *   was loaded.
+   */
+  protected function postLoad(&$queried_entities, $revision_id = FALSE) {
+    $class = isset($this->entityInfo['class']) ? $this->entityInfo['class']: $this->entityClass;
+    $class::postLoad($this, $queried_entities, $revision_id);
+  }
+
 }
diff --git a/core/lib/Drupal/Core/Entity/EntityStorageControllerInterface.php b/core/lib/Drupal/Core/Entity/EntityStorageControllerInterface.php
index c7f72f7..075b87f 100644
--- a/core/lib/Drupal/Core/Entity/EntityStorageControllerInterface.php
+++ b/core/lib/Drupal/Core/Entity/EntityStorageControllerInterface.php
@@ -171,4 +171,18 @@ public function invokeFieldMethod($method, EntityInterface $entity);
    */
   public function invokeFieldItemPrepareCache(EntityInterface $entity);
 
+  /**
+   * Returns the current entity type.
+   *
+   * @return string
+   */
+  public function entityType();
+
+  /**
+   * Returns the current entity info.
+   *
+   * @return string
+   */
+  public function entityInfo();
+
 }
diff --git a/core/modules/aggregator/lib/Drupal/aggregator/Entity/Feed.php b/core/modules/aggregator/lib/Drupal/aggregator/Entity/Feed.php
index 46b3894..47710d2 100644
--- a/core/modules/aggregator/lib/Drupal/aggregator/Entity/Feed.php
+++ b/core/modules/aggregator/lib/Drupal/aggregator/Entity/Feed.php
@@ -199,6 +199,14 @@ public static function preCreate(EntityStorageControllerInterface $storage_contr
   /**
    * {@inheritdoc}
    */
+  public static function postLoad(EntityStorageControllerInterface $storage_controller, array $entities, $revision_id = FALSE) {
+    $storage_controller->loadCategories($entities);
+    parent::postLoad($storage_controller, $entities, $revision_id = FALSE);
+  }
+
+  /**
+   * {@inheritdoc}
+   */
   public static function preDelete(EntityStorageControllerInterface $storage_controller, array $entities) {
     $storage_controller->deleteCategories($entities);
     foreach ($entities as $entity) {
diff --git a/core/modules/aggregator/lib/Drupal/aggregator/Entity/Item.php b/core/modules/aggregator/lib/Drupal/aggregator/Entity/Item.php
index f0b6694..f79762f 100644
--- a/core/modules/aggregator/lib/Drupal/aggregator/Entity/Item.php
+++ b/core/modules/aggregator/lib/Drupal/aggregator/Entity/Item.php
@@ -135,6 +135,14 @@ public function label($langcode = NULL) {
   /**
    * {@inheritdoc}
    */
+  public static function postLoad(EntityStorageControllerInterface $storage_controller, $entities, $revision_id = FALSE) {
+    $storage_controller->loadCategories($entities);
+    parent::attachLoad($storage_controller, $entities, $revision_id);
+  }
+
+  /**
+   * {@inheritdoc}
+   */
   public function postCreate(EntityStorageControllerInterface $storage_controller) {
     parent::postCreate($storage_controller);
 
diff --git a/core/modules/aggregator/lib/Drupal/aggregator/FeedStorageController.php b/core/modules/aggregator/lib/Drupal/aggregator/FeedStorageController.php
index 6db9b3f..b8f3eda 100644
--- a/core/modules/aggregator/lib/Drupal/aggregator/FeedStorageController.php
+++ b/core/modules/aggregator/lib/Drupal/aggregator/FeedStorageController.php
@@ -20,14 +20,6 @@
 class FeedStorageController extends DatabaseStorageControllerNG implements FeedStorageControllerInterface {
 
   /**
-   * Overrides Drupal\Core\Entity\DataBaseStorageController::attachLoad().
-   */
-  protected function attachLoad(&$queried_entities, $load_revision = FALSE) {
-    parent::attachLoad($queried_entities, $load_revision);
-    $this->loadCategories($queried_entities);
-  }
-
-  /**
    * {@inheritdoc}
    */
   public function loadCategories(array $feeds) {
diff --git a/core/modules/aggregator/lib/Drupal/aggregator/ItemStorageController.php b/core/modules/aggregator/lib/Drupal/aggregator/ItemStorageController.php
index e17c82b..a4e4da8 100644
--- a/core/modules/aggregator/lib/Drupal/aggregator/ItemStorageController.php
+++ b/core/modules/aggregator/lib/Drupal/aggregator/ItemStorageController.php
@@ -21,14 +21,6 @@
 class ItemStorageController extends DatabaseStorageControllerNG implements ItemStorageControllerInterface {
 
   /**
-   * Overrides Drupal\Core\Entity\DataBaseStorageController::attachLoad().
-   */
-  protected function attachLoad(&$queried_entities, $load_revision = FALSE) {
-    parent::attachLoad($queried_entities, $load_revision);
-    $this->loadCategories($queried_entities);
-  }
-
-  /**
    * {@inheritdoc}
    */
   public function loadCategories(array $entities) {
diff --git a/core/modules/block/custom_block/lib/Drupal/custom_block/CustomBlockStorageController.php b/core/modules/block/custom_block/lib/Drupal/custom_block/CustomBlockStorageController.php
deleted file mode 100644
index 9c5eb21..0000000
--- a/core/modules/block/custom_block/lib/Drupal/custom_block/CustomBlockStorageController.php
+++ /dev/null
@@ -1,39 +0,0 @@
-<?php
-
-/**
- * @file
- * Contains \Drupal\custom_block\CustomBlockStorageController.
- */
-
-namespace Drupal\custom_block;
-
-use Drupal\Core\Entity\DatabaseStorageControllerNG;
-use Drupal\Core\Entity\EntityStorageControllerInterface;
-
-/**
- * Controller class for custom blocks.
- *
- * This extends the Drupal\Core\Entity\DatabaseStorageControllerNG class,
- * adding required special handling for custom block entities.
- */
-class CustomBlockStorageController extends DatabaseStorageControllerNG {
-
-  /**
-   * Overrides \Drupal\Core\Entity\DatabaseStorageController::attachLoad().
-   */
-  protected function attachLoad(&$blocks, $load_revision = FALSE) {
-    // Create an array of block types for passing as a load argument.
-    // Note that blocks at this point are still \StdClass objects returned from
-    // the database.
-    foreach ($blocks as $id => $entity) {
-      $types[$entity->type] = $entity->type;
-    }
-
-    // Besides the list of blocks, pass one additional argument to
-    // hook_custom_block_load(), containing a list of block types that were
-    // loaded.
-    $this->hookLoadArguments = array($types);
-    parent::attachLoad($blocks, $load_revision);
-  }
-
-}
diff --git a/core/modules/block/custom_block/lib/Drupal/custom_block/Entity/CustomBlock.php b/core/modules/block/custom_block/lib/Drupal/custom_block/Entity/CustomBlock.php
index 2044050..7aadb99 100644
--- a/core/modules/block/custom_block/lib/Drupal/custom_block/Entity/CustomBlock.php
+++ b/core/modules/block/custom_block/lib/Drupal/custom_block/Entity/CustomBlock.php
@@ -22,7 +22,7 @@
  *   bundle_label = @Translation("Custom Block type"),
  *   module = "custom_block",
  *   controllers = {
- *     "storage" = "Drupal\custom_block\CustomBlockStorageController",
+ *     "storage" = "Drupal\Core\Entity\DatabaseStorageControllerNG",
  *     "access" = "Drupal\custom_block\CustomBlockAccessController",
  *     "list" = "Drupal\custom_block\CustomBlockListController",
  *     "render" = "Drupal\custom_block\CustomBlockRenderController",
@@ -308,4 +308,15 @@ public function getChangedTime() {
     return $this->get('changed')->value;
   }
 
+  /**
+   * {@inheritdoc}
+   */
+  protected static function hookLoadArguments(array $blocks) {
+    $types = array();
+    // Create an array of block types for passing as a load argument.
+    foreach ($blocks as $id => $entity) {
+      $types[$entity->bundle()] = $entity->bundle();
+    }
+    return array($types);
+  }
 }
diff --git a/core/modules/comment/lib/Drupal/comment/CommentStorageController.php b/core/modules/comment/lib/Drupal/comment/CommentStorageController.php
index f1c9c3f..d26d80b 100644
--- a/core/modules/comment/lib/Drupal/comment/CommentStorageController.php
+++ b/core/modules/comment/lib/Drupal/comment/CommentStorageController.php
@@ -40,14 +40,14 @@ protected function buildQuery($ids, $revision_id = FALSE) {
   /**
    * {@inheritdoc}
    */
-  protected function attachLoad(&$records, $load_revision = FALSE) {
+  protected function postLoad(&$records, $revision_id = FALSE) {
     // Prepare standard comment fields.
     foreach ($records as $key => $record) {
       $record->name = $record->uid ? $record->registered_name : $record->name;
       $record->node_type = 'comment_node_' . $record->node_type;
       $records[$key] = $record;
     }
-    parent::attachLoad($records, $load_revision);
+    parent::postLoad($records, $revision_id);
   }
 
   /**
diff --git a/core/modules/node/lib/Drupal/node/Entity/Node.php b/core/modules/node/lib/Drupal/node/Entity/Node.php
index 9071cac..906dc6f 100644
--- a/core/modules/node/lib/Drupal/node/Entity/Node.php
+++ b/core/modules/node/lib/Drupal/node/Entity/Node.php
@@ -77,6 +77,27 @@ public function getRevisionId() {
   /**
    * {@inheritdoc}
    */
+  protected static function hookLoadArguments(array $nodes) {
+    $bundles = array();
+    foreach ($nodes as $id => $node) {
+      $bundles[$node->bundle()] = TRUE;
+    }
+    return array(array_keys($bundles));
+  }
+
+  /**
+   * {@inheritdoc}
+   */
+  public static function preCreate(EntityStorageControllerInterface $storage_controller, array &$values) {
+    // @todo Handle this through property defaults.
+    if (empty($values['created'])) {
+      $values['created'] = REQUEST_TIME;
+    }
+  }
+
+  /**
+   * {@inheritdoc}
+   */
   public function preSave(EntityStorageControllerInterface $storage_controller) {
     parent::preSave($storage_controller);
 
@@ -142,6 +163,13 @@ public static function preDelete(EntityStorageControllerInterface $storage_contr
   /**
    * {@inheritdoc}
    */
+  public static function postDelete(EntityStorageControllerInterface $storage_controller, array $nodes) {
+    \Drupal::service('node.grant_storage')->deleteNodeRecords(array_keys($nodes));
+  }
+
+  /**
+   * {@inheritdoc}
+   */
   public function getType() {
     return $this->bundle();
   }
diff --git a/core/modules/node/lib/Drupal/node/NodeGrantDatabaseStorage.php b/core/modules/node/lib/Drupal/node/NodeGrantDatabaseStorage.php
index fba9847..50d0855 100644
--- a/core/modules/node/lib/Drupal/node/NodeGrantDatabaseStorage.php
+++ b/core/modules/node/lib/Drupal/node/NodeGrantDatabaseStorage.php
@@ -230,7 +230,7 @@ public function write(NodeInterface $node, array $grants, $realm = NULL, $delete
    * {@inheritdoc}
    */
   public function delete() {
-    $this->database->delete('node_access')->execute();
+    $this->database->truncate('node_access')->execute();
   }
 
   /**
@@ -256,4 +256,14 @@ public function count() {
     return $this->database->query('SELECT COUNT(*) FROM {node_access}')->fetchField();
   }
 
+  /**
+   * {@inheritdoc}
+   */
+  public function deleteNodeRecords(array $nids) {
+    $this->database->delete('node_access')
+      ->condition('nid', $nids, 'IN')
+      ->execute();
+
+  }
+
 }
diff --git a/core/modules/node/lib/Drupal/node/NodeGrantDatabaseStorageInterface.php b/core/modules/node/lib/Drupal/node/NodeGrantDatabaseStorageInterface.php
index 2457b8f..ad45741 100644
--- a/core/modules/node/lib/Drupal/node/NodeGrantDatabaseStorageInterface.php
+++ b/core/modules/node/lib/Drupal/node/NodeGrantDatabaseStorageInterface.php
@@ -121,4 +121,13 @@ public function access(EntityInterface $node, $operation, $langcode, AccountInte
    */
   public function count();
 
+  /**
+   * Remove the access records belonging to certain nodes.
+   *
+   * @param array $nids
+   *   A list of node IDs. The grant records belonging to these nodes will be
+   *   deleted.
+   */
+  public function deleteNodeRecords(array $nids);
+
 }
diff --git a/core/modules/node/lib/Drupal/node/NodeStorageController.php b/core/modules/node/lib/Drupal/node/NodeStorageController.php
index 1360af3..d68a3da 100644
--- a/core/modules/node/lib/Drupal/node/NodeStorageController.php
+++ b/core/modules/node/lib/Drupal/node/NodeStorageController.php
@@ -19,58 +19,6 @@
 class NodeStorageController extends DatabaseStorageControllerNG {
 
   /**
-   * Overrides Drupal\Core\Entity\DatabaseStorageController::create().
-   */
-  public function create(array $values) {
-    // @todo Handle this through property defaults.
-    if (empty($values['created'])) {
-      $values['created'] = REQUEST_TIME;
-    }
-    return parent::create($values);
-  }
-
-  /**
-   * Overrides Drupal\Core\Entity\DatabaseStorageControllerNG::attachLoad().
-   */
-  protected function attachLoad(&$queried_entities, $load_revision = FALSE) {
-    $queried_entities = $this->mapFromStorageRecords($queried_entities, $load_revision);
-
-    // Create an array of nodes for each content type and pass this to the
-    // object type specific callback. To preserve backward-compatibility we
-    // pass on BC decorators to node-specific hooks, while we pass on the
-    // regular entity objects else.
-    $typed_nodes = array();
-    foreach ($queried_entities as $id => $node) {
-      $typed_nodes[$node->bundle()][$id] = $queried_entities[$id];
-    }
-
-    if ($load_revision) {
-      $this->loadFieldItems($queried_entities, static::FIELD_LOAD_REVISION);
-    }
-    else {
-      $this->loadFieldItems($queried_entities, static::FIELD_LOAD_CURRENT);
-    }
-
-    // Besides the list of nodes, pass one additional argument to
-    // hook_node_load(), containing a list of node types that were loaded.
-    $argument = array_keys($typed_nodes);
-    $this->hookLoadArguments = array($argument);
-
-    // Call hook_entity_load().
-    foreach (\Drupal::moduleHandler()->getImplementations('entity_load') as $module) {
-      $function = $module . '_entity_load';
-      $function($queried_entities, $this->entityType);
-    }
-    // Call hook_TYPE_load(). The first argument for hook_TYPE_load() are
-    // always the queried entities, followed by additional arguments set in
-    // $this->hookLoadArguments.
-    $args = array_merge(array($queried_entities), $this->hookLoadArguments);
-    foreach (\Drupal::moduleHandler()->getImplementations($this->entityType . '_load') as $module) {
-      call_user_func_array($module . '_' . $this->entityType . '_load', $args);
-    }
-  }
-
-  /**
    * {@inheritdoc}
    */
   protected function mapToDataStorageRecord(EntityInterface $entity, $langcode) {
@@ -79,17 +27,4 @@ protected function mapToDataStorageRecord(EntityInterface $entity, $langcode) {
     $record->comment = isset($record->comment) ? intval($record->comment) : 0;
     return $record;
   }
-
-  /**
-   * Overrides Drupal\Core\Entity\DatabaseStorageController::postDelete().
-   */
-  protected function postDelete($nodes) {
-    // Delete values from other tables also referencing this node.
-    $ids = array_keys($nodes);
-
-    db_delete('node_access')
-      ->condition('nid', $ids, 'IN')
-      ->execute();
-  }
-
 }
diff --git a/core/modules/shortcut/lib/Drupal/shortcut/Entity/ShortcutSet.php b/core/modules/shortcut/lib/Drupal/shortcut/Entity/ShortcutSet.php
index b52fccb..b3d934d 100644
--- a/core/modules/shortcut/lib/Drupal/shortcut/Entity/ShortcutSet.php
+++ b/core/modules/shortcut/lib/Drupal/shortcut/Entity/ShortcutSet.php
@@ -102,6 +102,19 @@ public function postCreate(EntityStorageControllerInterface $storage_controller)
   /**
    * {@inheritdoc}
    */
+  public static function postLoad(EntityStorageControllerInterface $storage_controller, array $entities, $revision_id = FALSE) {
+    foreach ($entities as $id => $entity) {
+      $links = menu_load_links('shortcut-' . $id);
+      foreach ($links as $menu_link) {
+        $entity->links[$menu_link->uuid()] = $menu_link;
+      }
+    }
+    parent::postLoad($storage_controller, $entities, $revision_id);
+  }
+
+  /**
+   * {@inheritdoc}
+   */
   public function preSave(EntityStorageControllerInterface $storage_controller) {
     parent::preSave($storage_controller);
 
diff --git a/core/modules/shortcut/lib/Drupal/shortcut/ShortcutSetStorageController.php b/core/modules/shortcut/lib/Drupal/shortcut/ShortcutSetStorageController.php
index 2f17850..7c0dee5 100644
--- a/core/modules/shortcut/lib/Drupal/shortcut/ShortcutSetStorageController.php
+++ b/core/modules/shortcut/lib/Drupal/shortcut/ShortcutSetStorageController.php
@@ -16,20 +16,6 @@
 class ShortcutSetStorageController extends ConfigStorageController implements ShortcutSetStorageControllerInterface {
 
   /**
-   * Overrides \Drupal\config\ConfigStorageController::attachLoad().
-   */
-  protected function attachLoad(&$queried_entities, $revision_id = FALSE) {
-    parent::attachLoad($queried_entities, $revision_id);
-
-    foreach ($queried_entities as $id => $entity) {
-      $links = menu_load_links('shortcut-' . $id);
-      foreach ($links as $menu_link) {
-        $entity->links[$menu_link->uuid()] = $menu_link;
-      }
-    }
-  }
-
-  /**
    * {@inheritdoc}
    */
   public function deleteAssignedShortcutSets(ShortcutSetInterface $entity) {
diff --git a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTest.php b/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTest.php
index 95d74ce..6769624 100644
--- a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTest.php
+++ b/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTest.php
@@ -10,6 +10,7 @@
 use Drupal\Core\Entity\EntityNG;
 use Drupal\Core\Entity\Annotation\EntityType;
 use Drupal\Core\Annotation\Translation;
+use Drupal\Core\Entity\EntityStorageControllerInterface;
 use Drupal\Core\Language\Language;
 
 /**
@@ -20,7 +21,7 @@
  *   label = @Translation("Test entity"),
  *   module = "entity_test",
  *   controllers = {
- *     "storage" = "Drupal\entity_test\EntityTestStorageController",
+ *     "storage" = "Drupal\Core\Entity\DatabaseStorageControllerNG",
  *     "list" = "Drupal\entity_test\EntityTestListController",
  *     "access" = "Drupal\entity_test\EntityTestAccessController",
  *     "form" = {
@@ -90,6 +91,15 @@ protected function init() {
   }
 
   /**
+   * {@inheritdoc}
+   */
+  public static function preCreate(EntityStorageControllerInterface $storage_controller, array &$values) {
+    if (empty($values['type'])) {
+      $values['type'] = $storage_controller->entityType();
+    }
+  }
+
+  /**
    * Overrides Drupal\entity\Entity::label().
    */
   public function label($langcode = Language::LANGCODE_DEFAULT) {
diff --git a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestCache.php b/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestCache.php
index b4a064f..c1c7068 100644
--- a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestCache.php
+++ b/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestCache.php
@@ -19,7 +19,7 @@
  *   label = @Translation("Test entity with field cache"),
  *   module = "entity_test",
  *   controllers = {
- *     "storage" = "Drupal\entity_test\EntityTestStorageController",
+ *     "storage" = "Drupal\Core\Entity\DatabaseStorageControllerNG",
  *     "access" = "Drupal\entity_test\EntityTestAccessController",
  *     "form" = {
  *       "default" = "Drupal\entity_test\EntityTestFormController"
diff --git a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestDefaultAccess.php b/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestDefaultAccess.php
index 4893c83..80d2cee 100644
--- a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestDefaultAccess.php
+++ b/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestDefaultAccess.php
@@ -18,7 +18,7 @@
  *   label = @Translation("Test entity with default access"),
  *   module = "entity_test",
  *   controllers = {
- *     "storage" = "Drupal\entity_test\EntityTestStorageController"
+ *     "storage" = "Drupal\Core\Entity\DatabaseStorageControllerNG"
  *   },
  *   base_table = "entity_test",
  *   entity_keys = {
diff --git a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestLabel.php b/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestLabel.php
index fc969c3..a53496e 100644
--- a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestLabel.php
+++ b/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestLabel.php
@@ -18,7 +18,7 @@
  *   label = @Translation("Entity Test label"),
  *   module = "entity_test",
  *   controllers = {
- *     "storage" = "Drupal\entity_test\EntityTestStorageController",
+ *     "storage" = "Drupal\Core\Entity\DatabaseStorageControllerNG",
  *     "render" = "Drupal\entity_test\EntityTestRenderController"
  *   },
  *   base_table = "entity_test",
diff --git a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestLabelCallback.php b/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestLabelCallback.php
index 1784aa2..f9b5812 100644
--- a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestLabelCallback.php
+++ b/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestLabelCallback.php
@@ -18,7 +18,7 @@
  *   label = @Translation("Entity test label callback"),
  *   module = "entity_test",
  *   controllers = {
- *     "storage" = "Drupal\entity_test\EntityTestStorageController"
+ *     "storage" = "Drupal\Core\Entity\DatabaseStorageControllerNG"
  *   },
  *   field_cache = FALSE,
  *   base_table = "entity_test",
diff --git a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestMul.php b/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestMul.php
index 3f7e833..363bc16 100644
--- a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestMul.php
+++ b/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestMul.php
@@ -19,7 +19,7 @@
  *   label = @Translation("Test entity - data table"),
  *   module = "entity_test",
  *   controllers = {
- *     "storage" = "Drupal\entity_test\EntityTestStorageController",
+ *     "storage" = "Drupal\Core\Entity\DatabaseStorageControllerNG",
  *     "access" = "Drupal\entity_test\EntityTestAccessController",
  *     "form" = {
  *       "default" = "Drupal\entity_test\EntityTestFormController"
diff --git a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestMulRev.php b/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestMulRev.php
index 4a578e2..4219ea3 100644
--- a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestMulRev.php
+++ b/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestMulRev.php
@@ -19,7 +19,7 @@
  *   label = @Translation("Test entity - revisions and data table"),
  *   module = "entity_test",
  *   controllers = {
- *     "storage" = "Drupal\entity_test\EntityTestStorageController",
+ *     "storage" = "Drupal\Core\Entity\DatabaseStorageControllerNG",
  *     "access" = "Drupal\entity_test\EntityTestAccessController",
  *     "form" = {
  *       "default" = "Drupal\entity_test\EntityTestFormController"
diff --git a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestNoLabel.php b/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestNoLabel.php
index f56a5e0..85dcea6 100644
--- a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestNoLabel.php
+++ b/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestNoLabel.php
@@ -18,7 +18,7 @@
  *   label = @Translation("Entity Test without label"),
  *   module = "entity_test",
  *   controllers = {
- *     "storage" = "Drupal\entity_test\EntityTestStorageController"
+ *     "storage" = "Drupal\Core\Entity\DatabaseStorageControllerNG"
  *   },
  *   field_cache = FALSE,
  *   base_table = "entity_test",
diff --git a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestRender.php b/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestRender.php
index 3c0869d..0083964 100644
--- a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestRender.php
+++ b/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestRender.php
@@ -18,7 +18,7 @@
  *   label = @Translation("Test render entity"),
  *   module = "entity_test",
  *   controllers = {
- *     "storage" = "Drupal\entity_test\EntityTestStorageController",
+ *     "storage" = "Drupal\Core\Entity\DatabaseStorageControllerNG",
  *     "render" = "Drupal\entity_test\EntityTestRenderController"
  *   },
  *   base_table = "entity_test",
diff --git a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestRev.php b/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestRev.php
index 0fabcca..41caab8 100644
--- a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestRev.php
+++ b/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestRev.php
@@ -19,7 +19,7 @@
  *   label = @Translation("Test entity - revisions"),
  *   module = "entity_test",
  *   controllers = {
- *     "storage" = "Drupal\entity_test\EntityTestStorageController",
+ *     "storage" = "Drupal\Core\Entity\DatabaseStorageControllerNG",
  *     "access" = "Drupal\entity_test\EntityTestAccessController",
  *     "form" = {
  *       "default" = "Drupal\entity_test\EntityTestFormController"
diff --git a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/EntityTestStorageController.php b/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/EntityTestStorageController.php
deleted file mode 100644
index bb8d390..0000000
--- a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/EntityTestStorageController.php
+++ /dev/null
@@ -1,30 +0,0 @@
-<?php
-
-/**
- * @file
- * Definition of Drupal\entity_test\EntityTestStorageController.
- */
-
-namespace Drupal\entity_test;
-
-use Drupal\Core\Entity\DatabaseStorageControllerNG;
-
-/**
- * Defines the controller class for the test entity.
- *
- * This extends the Drupal\Core\Entity\DatabaseStorageController class, adding
- * required special handling for test entities.
- */
-class EntityTestStorageController extends DatabaseStorageControllerNG {
-
-  /**
-   * {@inheritdoc}
-   */
-  public function create(array $values) {
-    if (empty($values['type'])) {
-      $values['type'] = $this->entityType;
-    }
-    return parent::create($values);
-  }
-
-}
diff --git a/core/modules/user/lib/Drupal/user/RoleStorageController.php b/core/modules/user/lib/Drupal/user/RoleStorageController.php
index 03937a6..9352140 100644
--- a/core/modules/user/lib/Drupal/user/RoleStorageController.php
+++ b/core/modules/user/lib/Drupal/user/RoleStorageController.php
@@ -27,12 +27,12 @@ public function deleteRoleReferences(array $rids) {
   /**
    * {@inheritdoc}
    */
-  protected function attachLoad(&$queried_entities, $revision_id = FALSE) {
+  protected function postLoad(&$queried_entities, $revision_id = FALSE) {
     // Sort the queried roles by their weight.
     // See \Drupal\Core\Config\Entity\ConfigEntityBase::sort().
     uasort($queried_entities, array($this->entityInfo['class'], 'sort'));
 
-    parent::attachLoad($queried_entities, $revision_id);
+    parent::postLoad($queried_entities, $revision_id);
   }
 
 }
diff --git a/core/modules/views/lib/Drupal/views/ViewStorageController.php b/core/modules/views/lib/Drupal/views/ViewStorageController.php
index 087d34e..decc571 100644
--- a/core/modules/views/lib/Drupal/views/ViewStorageController.php
+++ b/core/modules/views/lib/Drupal/views/ViewStorageController.php
@@ -33,12 +33,12 @@ public function loadMultiple(array $ids = NULL) {
   /**
    * {@inheritdoc}
    */
-  protected function attachLoad(&$queried_entities, $revision_id = FALSE) {
+  protected function postLoad(&$queried_entities, $revision_id = FALSE) {
     foreach ($queried_entities as $entity) {
       $entity->mergeDefaultDisplaysOptions();
     }
 
-    parent::attachLoad($queried_entities, $revision_id);
+    parent::postLoad($queried_entities, $revision_id);
   }
 
 
diff --git a/core/modules/views_ui/lib/Drupal/views_ui/ViewUI.php b/core/modules/views_ui/lib/Drupal/views_ui/ViewUI.php
index 7d88791..7c3c42f 100644
--- a/core/modules/views_ui/lib/Drupal/views_ui/ViewUI.php
+++ b/core/modules/views_ui/lib/Drupal/views_ui/ViewUI.php
@@ -1199,7 +1199,7 @@ public static function postDelete(EntityStorageControllerInterface $storage_cont
   /**
    * {@inheritdoc}
    */
-  public static function postLoad(EntityStorageControllerInterface $storage_controller, array $entities) {
+  public static function postLoad(EntityStorageControllerInterface $storage_controller, array $entities, $revision_id = FALSE) {
   }
 
   /**

diff --git a/core/lib/Drupal/Core/Config/Entity/ConfigStorageController.php b/core/lib/Drupal/Core/Config/Entity/ConfigStorageController.php
index 2fd44c4..28c4be8 100644
--- a/core/lib/Drupal/Core/Config/Entity/ConfigStorageController.php
+++ b/core/lib/Drupal/Core/Config/Entity/ConfigStorageController.php
@@ -141,11 +141,11 @@ public function loadMultiple(array $ids = NULL) {
       $queried_entities = $this->buildQuery($ids);
     }
 
     // Pass all entities loaded from the database through $this->attachLoad(),
     // which calls the
     // entity type specific load callback, for example hook_node_type_load().
     if (!empty($queried_entities)) {
-      $this->attachLoad($queried_entities);
+      $this->postLoad($queried_entities);
       $entities += $queried_entities;
     }
 
@@ -286,38 +286,6 @@ protected function buildQuery($ids, $revision_id = FALSE) {
   }
 
   /**
-   * Attaches data to entities upon loading.
-   *
-   * This will attach fields, if the entity is fieldable. It calls
-   * hook_entity_load() for modules which need to add data to all entities.
-   * It also calls hook_TYPE_load() on the loaded entities. For example
-   * hook_node_load() or hook_user_load(). If your hook_TYPE_load()
-   * expects special parameters apart from the queried entities, you can set
-   * $this->hookLoadArguments prior to calling the method.
-   * See Drupal\node\NodeStorageController::attachLoad() for an example.
-   *
-   * @param $queried_entities
-   *   Associative array of query results, keyed on the entity ID.
-   * @param $revision_id
-   *   ID of the revision that was loaded, or FALSE if the most current revision
-   *   was loaded.
-   */
-  protected function attachLoad(&$queried_entities, $revision_id = FALSE) {
-    // Call hook_entity_load().
-    foreach (\Drupal::moduleHandler()->getImplementations('entity_load') as $module) {
-      $function = $module . '_entity_load';
-      $function($queried_entities, $this->entityType);
-    }
-    // Call hook_TYPE_load(). The first argument for hook_TYPE_load() are
-    // always the queried entities, followed by additional arguments set in
-    // $this->hookLoadArguments.
-    $args = array_merge(array($queried_entities), $this->hookLoadArguments);
-    foreach (\Drupal::moduleHandler()->getImplementations($this->entityType . '_load') as $module) {
-      call_user_func_array($module . '_' . $this->entityType . '_load', $args);
-    }
-  }
-
-  /**
    * Implements Drupal\Core\Entity\EntityStorageControllerInterface::create().
    */
   public function create(array $values) {
@@ -141,11 +141,11 @@ public function loadMultiple(array $ids = NULL) {
     }
 
-    // Pass all entities loaded from the database through $this->attachLoad(),
+    // Pass all entities loaded from the database through $this->postLoad(),
     // which calls the
     // entity type specific load callback, for example hook_node_type_load().
diff --git a/core/lib/Drupal/Core/Entity/DatabaseStorageController.php b/core/lib/Drupal/Core/Entity/DatabaseStorageController.php
index 4bd89ae..91e4eda 100644
--- a/core/lib/Drupal/Core/Entity/DatabaseStorageController.php
+++ b/core/lib/Drupal/Core/Entity/DatabaseStorageController.php
@@ -174,11 +174,11 @@ public function loadMultiple(array $ids = NULL) {
       $queried_entities = $query_result->fetchAllAssoc($this->idKey);
     }
 
     // Pass all entities loaded from the database through $this->attachLoad(),
     // which attaches fields (if supported by the entity type) and calls the
     // entity type specific load callback, for example hook_node_load().
     if (!empty($queried_entities)) {
-      $this->attachLoad($queried_entities);
+      $this->postLoad($queried_entities);
       $entities += $queried_entities;
     }
 
@@ -226,11 +226,11 @@ public function loadRevision($revision_id) {
     }
     $queried_entities = $query_result->fetchAllAssoc($this->idKey);
 
     // Pass the loaded entities from the database through $this->attachLoad(),
     // which attaches fields (if supported by the entity type) and calls the
     // entity type specific load callback, for example hook_node_load().
     if (!empty($queried_entities)) {
-      $this->attachLoad($queried_entities, $revision_id);
+      $this->postLoad($queried_entities, $revision_id);
     }
     return reset($queried_entities);
   }
@@ -348,37 +348,21 @@ protected function buildQuery($ids, $revision_id = FALSE) {
   /**
    * Attaches data to entities upon loading.
    *
-   * This will attach fields, if the entity is fieldable. It calls
-   * hook_entity_load() for modules which need to add data to all entities.
-   * It also calls hook_TYPE_load() on the loaded entities. For example
-   * hook_node_load() or hook_user_load(). If your hook_TYPE_load()
-   * expects special parameters apart from the queried entities, you can set
-   * $this->hookLoadArguments prior to calling the method.
-   * See Drupal\node\NodeStorageController::attachLoad() for an example.
+   * This will attach fields, if the entity is fieldable and calls the
+   * entity postLoad() method.
    *
    * @param $queried_entities
    *   Associative array of query results, keyed on the entity ID.
-   * @param $load_revision
-   *   (optional) TRUE if the revision should be loaded, defaults to FALSE.
+   * @param $revision_id
+   *   ID of the revision that was loaded, or FALSE if the most current revision
+   *   was loaded.
    */
-  protected function attachLoad(&$queried_entities, $load_revision = FALSE) {
+  protected function postLoad(array &$queried_entities, $revision_id = FALSE) {
     // Attach field values.
     if ($this->entityInfo['fieldable']) {
-      $this->loadFieldItems($queried_entities, $load_revision ? static::FIELD_LOAD_REVISION : static::FIELD_LOAD_CURRENT);
-    }
-
-    // Call hook_entity_load().
-    foreach (\Drupal::moduleHandler()->getImplementations('entity_load') as $module) {
-      $function = $module . '_entity_load';
-      $function($queried_entities, $this->entityType);
-    }
-    // Call hook_TYPE_load(). The first argument for hook_TYPE_load() are
-    // always the queried entities, followed by additional arguments set in
-    // $this->hookLoadArguments.
-    $args = array_merge(array($queried_entities), $this->hookLoadArguments);
-    foreach (\Drupal::moduleHandler()->getImplementations($this->entityType . '_load') as $module) {
-      call_user_func_array($module . '_' . $this->entityType . '_load', $args);
+      $this->loadFieldItems($queried_entities, $revision_id ? static::FIELD_LOAD_REVISION : static::FIELD_LOAD_CURRENT);
     }
+    parent::postLoad($queried_entities, $revision_id);
   }
 
   /**
@@ -174,11 +174,11 @@ public function loadMultiple(array $ids = NULL) {
     }
 
-    // Pass all entities loaded from the database through $this->attachLoad(),
+    // Pass all entities loaded from the database through $this->postLoad(),
     // which attaches fields (if supported by the entity type) and calls the
     // entity type specific load callback, for example hook_node_load().
@@ -226,11 +226,11 @@ public function loadRevision($revision_id) {
     $queried_entities = $query_result->fetchAllAssoc($this->idKey);
 
-    // Pass the loaded entities from the database through $this->attachLoad(),
+    // Pass the loaded entities from the database through $this->postLoad(),
     // which attaches fields (if supported by the entity type) and calls the
     // entity type specific load callback, for example hook_node_load().
diff --git a/core/modules/menu_link/lib/Drupal/menu_link/MenuLinkStorageController.php b/core/modules/menu_link/lib/Drupal/menu_link/MenuLinkStorageController.php
index 1fa6140..1390d45 100644
--- a/core/modules/menu_link/lib/Drupal/menu_link/MenuLinkStorageController.php
+++ b/core/modules/menu_link/lib/Drupal/menu_link/MenuLinkStorageController.php
@@ -109,12 +109,12 @@ protected function buildQuery($ids, $revision_id = FALSE) {
   }
 
   /**
-   * Overrides DatabaseStorageController::attachLoad().
+   * Overrides DatabaseStorageController::postLoad().
    *
    * @todo Don't call parent::attachLoad() at all because we want to be able to
    * control the entity load hooks.
    */
-  protected function attachLoad(&$menu_links, $load_revision = FALSE) {
+  protected function postLoad(&$menu_links, $load_revision = FALSE) {
     $routes = array();
 
     foreach ($menu_links as &$menu_link) {
@@ -145,7 +145,7 @@ protected function attachLoad(&$menu_links, $load_revision = FALSE) {
       }
     }
 
-    parent::attachLoad($menu_links, $load_revision);
+    parent::postLoad($menu_links, $load_revision);
   }
 
   /**
@@ -109,12 +109,12 @@ protected function buildQuery($ids, $revision_id = FALSE) {
    *
-   * @todo Don't call parent::attachLoad() at all because we want to be able to
+   * @todo Don't call parent::postLoad() at all because we want to be able to
    * control the entity load hooks.
    */
diff --git a/core/modules/user/lib/Drupal/user/UserStorageController.php b/core/modules/user/lib/Drupal/user/UserStorageController.php
index 03d32b1..15a4123 100644
--- a/core/modules/user/lib/Drupal/user/UserStorageController.php
+++ b/core/modules/user/lib/Drupal/user/UserStorageController.php
@@ -79,9 +79,9 @@ public static function createInstance(ContainerInterface $container, $entity_typ
   }
 
   /**
-   * Overrides Drupal\Core\Entity\DatabaseStorageController::attachLoad().
+   * Overrides Drupal\Core\Entity\DatabaseStorageController::postLoad().
    */
-  function attachLoad(&$queried_users, $load_revision = FALSE) {
+  function postLoad(&$queried_users, $load_revision = FALSE) {
     foreach ($queried_users as $key => $record) {
       $queried_users[$key]->roles = array();
       if ($record->uid) {
@@ -95,9 +95,9 @@ function attachLoad(&$queried_users, $load_revision = FALSE) {
     // Add any additional roles from the database.
     $this->addRoles($queried_users);
 
     // Call the default attachLoad() method. This will add fields and call
     // hook_user_load().
-    parent::attachLoad($queried_users, $load_revision);
+    parent::postLoad($queried_users, $load_revision);
   }
 
   /**
@@ -95,9 +95,9 @@ function attachLoad(&$queried_users, $load_revision = FALSE) {
     $this->addRoles($queried_users);
 
-    // Call the default attachLoad() method. This will add fields and call
+    // Call the default postLoad() method. This will add fields and call
     // hook_user_load().